pub struct ForwardKeys {
pub rho: [u8; 32],
pub mu: [u8; 32],
}Expand description
Keys used to forward the onion packet.
Fields
rho: [u8; 32]Key derived from the shared secret for the hop. It is used to encrypt the packet data.
mu: [u8; 32]Key derived from the shared secret for the hop. It is used to compute the HMAC of the packet data.
Implementations
Sourceimpl ForwardKeys
impl ForwardKeys
Sourcepub fn new(shared_secret: &[u8]) -> ForwardKeys
pub fn new(shared_secret: &[u8]) -> ForwardKeys
Derive keys for forwarding the onion packet from the shared secret.